/*----------------------------------------------------------------------- 
 * 
 * File Name: VectorFactories.c
 * 
 * Author: Finn, L. S.
 * 
 * Revision: $Id: VectorFactories.m4,v 1.1 2000/04/17 23:22:47 jolien Exp $
 * 
 *----------------------------------------------------------------------- 
 * 
 * SYNOPSIS 
 * void TYPECODECreateVector( Status *, TYPEVector **vector, UINT4 length );
 * void TYPECODEDestroyVector( Status *, TYPEVector **vector);
 * 
 * DESCRIPTION 
 * Create/destroy a TYPEVector object. 
 * 
 * DIAGNOSTICS 
 * Illegal length, vector == NULL, *vector != NULL, malloc failure
 *
 * CALLS
 * LALMalloc
 * 
 *-----------------------------------------------------------------------
 */

#include "LALStdlib.h"
#include "AVFactories.h"

NRCSID( VECTORFACTORIESC, "$Id: VectorFactories.m4,v 1.1 2000/04/17 23:22:47 jolien Exp $" );

















void ZCreateVector ( Status *status, COMPLEX16Vector **vector, UINT4 length ) 
{
  /* 
   * Initialize status structure
   */

  INITSTATUS( status, "ZCreateVector", VECTORFACTORIESC );	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT( length > 0, status, CREATEVECTOR_ELENGTH, CREATEVECTOR_MSGELENGTH );

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT( vector != NULL, status, CREATEVECTOR_EVPTR, CREATEVECTOR_MSGEVPTR );

  ASSERT( *vector == NULL, status, CREATEVECTOR_EUPTR, CREATEVECTOR_MSGEUPTR );

  /*
   * Allocate pointer
   */

  *vector = ( COMPLEX16Vector * ) LALMalloc( sizeof( COMPLEX16Vector ) );
  ASSERT( *vector != NULL, status,
          CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC );

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = ( COMPLEX16 * ) LALMalloc( length*sizeof( COMPLEX16 ) );

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree( *vector );
    ABORT( status, CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC );
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN( status );
}
















void ZDestroyVector ( Status *status, COMPLEX16Vector **vector )
{
  /* 
   * Initialize status
   */

  INITSTATUS( status, "ZDestroyVector", VECTORFACTORIESC );	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT( vector != NULL, status, DESTROYVECTOR_EVPTR, DESTROYVECTOR_MSGEVPTR );

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT( *vector != NULL, status, DESTROYVECTOR_EUPTR, DESTROYVECTOR_MSGEUPTR );

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT( (*vector)->data != NULL, status,
          DESTROYVECTOR_EDPTR, DESTROYVECTOR_MSGEDPTR );

  /* Ok, now let's free allocated storage */

  LALFree( (*vector)->data ); /* free allocated data */
  LALFree( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN( status );
}


















void CCreateVector ( Status *status, COMPLEX8Vector **vector, UINT4 length ) 
{
  /* 
   * Initialize status structure
   */

  INITSTATUS( status, "CCreateVector", VECTORFACTORIESC );	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT( length > 0, status, CREATEVECTOR_ELENGTH, CREATEVECTOR_MSGELENGTH );

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT( vector != NULL, status, CREATEVECTOR_EVPTR, CREATEVECTOR_MSGEVPTR );

  ASSERT( *vector == NULL, status, CREATEVECTOR_EUPTR, CREATEVECTOR_MSGEUPTR );

  /*
   * Allocate pointer
   */

  *vector = ( COMPLEX8Vector * ) LALMalloc( sizeof( COMPLEX8Vector ) );
  ASSERT( *vector != NULL, status,
          CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC );

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = ( COMPLEX8 * ) LALMalloc( length*sizeof( COMPLEX8 ) );

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree( *vector );
    ABORT( status, CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC );
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN( status );
}
















void CDestroyVector ( Status *status, COMPLEX8Vector **vector )
{
  /* 
   * Initialize status
   */

  INITSTATUS( status, "CDestroyVector", VECTORFACTORIESC );	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT( vector != NULL, status, DESTROYVECTOR_EVPTR, DESTROYVECTOR_MSGEVPTR );

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT( *vector != NULL, status, DESTROYVECTOR_EUPTR, DESTROYVECTOR_MSGEUPTR );

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT( (*vector)->data != NULL, status,
          DESTROYVECTOR_EDPTR, DESTROYVECTOR_MSGEDPTR );

  /* Ok, now let's free allocated storage */

  LALFree( (*vector)->data ); /* free allocated data */
  LALFree( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN( status );
}


















void DCreateVector ( Status *status, REAL8Vector **vector, UINT4 length ) 
{
  /* 
   * Initialize status structure
   */

  INITSTATUS( status, "DCreateVector", VECTORFACTORIESC );	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT( length > 0, status, CREATEVECTOR_ELENGTH, CREATEVECTOR_MSGELENGTH );

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT( vector != NULL, status, CREATEVECTOR_EVPTR, CREATEVECTOR_MSGEVPTR );

  ASSERT( *vector == NULL, status, CREATEVECTOR_EUPTR, CREATEVECTOR_MSGEUPTR );

  /*
   * Allocate pointer
   */

  *vector = ( REAL8Vector * ) LALMalloc( sizeof( REAL8Vector ) );
  ASSERT( *vector != NULL, status,
          CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC );

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = ( REAL8 * ) LALMalloc( length*sizeof( REAL8 ) );

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree( *vector );
    ABORT( status, CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC );
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN( status );
}
















void DDestroyVector ( Status *status, REAL8Vector **vector )
{
  /* 
   * Initialize status
   */

  INITSTATUS( status, "DDestroyVector", VECTORFACTORIESC );	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT( vector != NULL, status, DESTROYVECTOR_EVPTR, DESTROYVECTOR_MSGEVPTR );

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT( *vector != NULL, status, DESTROYVECTOR_EUPTR, DESTROYVECTOR_MSGEUPTR );

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT( (*vector)->data != NULL, status,
          DESTROYVECTOR_EDPTR, DESTROYVECTOR_MSGEDPTR );

  /* Ok, now let's free allocated storage */

  LALFree( (*vector)->data ); /* free allocated data */
  LALFree( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN( status );
}


















void SCreateVector ( Status *status, REAL4Vector **vector, UINT4 length ) 
{
  /* 
   * Initialize status structure
   */

  INITSTATUS( status, "SCreateVector", VECTORFACTORIESC );	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT( length > 0, status, CREATEVECTOR_ELENGTH, CREATEVECTOR_MSGELENGTH );

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT( vector != NULL, status, CREATEVECTOR_EVPTR, CREATEVECTOR_MSGEVPTR );

  ASSERT( *vector == NULL, status, CREATEVECTOR_EUPTR, CREATEVECTOR_MSGEUPTR );

  /*
   * Allocate pointer
   */

  *vector = ( REAL4Vector * ) LALMalloc( sizeof( REAL4Vector ) );
  ASSERT( *vector != NULL, status,
          CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC );

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = ( REAL4 * ) LALMalloc( length*sizeof( REAL4 ) );

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree( *vector );
    ABORT( status, CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC );
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN( status );
}
















void SDestroyVector ( Status *status, REAL4Vector **vector )
{
  /* 
   * Initialize status
   */

  INITSTATUS( status, "SDestroyVector", VECTORFACTORIESC );	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT( vector != NULL, status, DESTROYVECTOR_EVPTR, DESTROYVECTOR_MSGEVPTR );

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT( *vector != NULL, status, DESTROYVECTOR_EUPTR, DESTROYVECTOR_MSGEUPTR );

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT( (*vector)->data != NULL, status,
          DESTROYVECTOR_EDPTR, DESTROYVECTOR_MSGEDPTR );

  /* Ok, now let's free allocated storage */

  LALFree( (*vector)->data ); /* free allocated data */
  LALFree( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN( status );
}


















void I2CreateVector ( Status *status, INT2Vector **vector, UINT4 length ) 
{
  /* 
   * Initialize status structure
   */

  INITSTATUS( status, "I2CreateVector", VECTORFACTORIESC );	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT( length > 0, status, CREATEVECTOR_ELENGTH, CREATEVECTOR_MSGELENGTH );

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT( vector != NULL, status, CREATEVECTOR_EVPTR, CREATEVECTOR_MSGEVPTR );

  ASSERT( *vector == NULL, status, CREATEVECTOR_EUPTR, CREATEVECTOR_MSGEUPTR );

  /*
   * Allocate pointer
   */

  *vector = ( INT2Vector * ) LALMalloc( sizeof( INT2Vector ) );
  ASSERT( *vector != NULL, status,
          CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC );

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = ( INT2 * ) LALMalloc( length*sizeof( INT2 ) );

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree( *vector );
    ABORT( status, CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC );
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN( status );
}
















void I2DestroyVector ( Status *status, INT2Vector **vector )
{
  /* 
   * Initialize status
   */

  INITSTATUS( status, "I2DestroyVector", VECTORFACTORIESC );	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT( vector != NULL, status, DESTROYVECTOR_EVPTR, DESTROYVECTOR_MSGEVPTR );

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT( *vector != NULL, status, DESTROYVECTOR_EUPTR, DESTROYVECTOR_MSGEUPTR );

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT( (*vector)->data != NULL, status,
          DESTROYVECTOR_EDPTR, DESTROYVECTOR_MSGEDPTR );

  /* Ok, now let's free allocated storage */

  LALFree( (*vector)->data ); /* free allocated data */
  LALFree( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN( status );
}


















void I4CreateVector ( Status *status, INT4Vector **vector, UINT4 length ) 
{
  /* 
   * Initialize status structure
   */

  INITSTATUS( status, "I4CreateVector", VECTORFACTORIESC );	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT( length > 0, status, CREATEVECTOR_ELENGTH, CREATEVECTOR_MSGELENGTH );

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT( vector != NULL, status, CREATEVECTOR_EVPTR, CREATEVECTOR_MSGEVPTR );

  ASSERT( *vector == NULL, status, CREATEVECTOR_EUPTR, CREATEVECTOR_MSGEUPTR );

  /*
   * Allocate pointer
   */

  *vector = ( INT4Vector * ) LALMalloc( sizeof( INT4Vector ) );
  ASSERT( *vector != NULL, status,
          CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC );

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = ( INT4 * ) LALMalloc( length*sizeof( INT4 ) );

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree( *vector );
    ABORT( status, CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC );
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN( status );
}
















void I4DestroyVector ( Status *status, INT4Vector **vector )
{
  /* 
   * Initialize status
   */

  INITSTATUS( status, "I4DestroyVector", VECTORFACTORIESC );	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT( vector != NULL, status, DESTROYVECTOR_EVPTR, DESTROYVECTOR_MSGEVPTR );

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT( *vector != NULL, status, DESTROYVECTOR_EUPTR, DESTROYVECTOR_MSGEUPTR );

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT( (*vector)->data != NULL, status,
          DESTROYVECTOR_EDPTR, DESTROYVECTOR_MSGEDPTR );

  /* Ok, now let's free allocated storage */

  LALFree( (*vector)->data ); /* free allocated data */
  LALFree( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN( status );
}


















void I8CreateVector ( Status *status, INT8Vector **vector, UINT4 length ) 
{
  /* 
   * Initialize status structure
   */

  INITSTATUS( status, "I8CreateVector", VECTORFACTORIESC );	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT( length > 0, status, CREATEVECTOR_ELENGTH, CREATEVECTOR_MSGELENGTH );

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT( vector != NULL, status, CREATEVECTOR_EVPTR, CREATEVECTOR_MSGEVPTR );

  ASSERT( *vector == NULL, status, CREATEVECTOR_EUPTR, CREATEVECTOR_MSGEUPTR );

  /*
   * Allocate pointer
   */

  *vector = ( INT8Vector * ) LALMalloc( sizeof( INT8Vector ) );
  ASSERT( *vector != NULL, status,
          CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC );

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = ( INT8 * ) LALMalloc( length*sizeof( INT8 ) );

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree( *vector );
    ABORT( status, CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC );
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN( status );
}
















void I8DestroyVector ( Status *status, INT8Vector **vector )
{
  /* 
   * Initialize status
   */

  INITSTATUS( status, "I8DestroyVector", VECTORFACTORIESC );	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT( vector != NULL, status, DESTROYVECTOR_EVPTR, DESTROYVECTOR_MSGEVPTR );

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT( *vector != NULL, status, DESTROYVECTOR_EUPTR, DESTROYVECTOR_MSGEUPTR );

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT( (*vector)->data != NULL, status,
          DESTROYVECTOR_EDPTR, DESTROYVECTOR_MSGEDPTR );

  /* Ok, now let's free allocated storage */

  LALFree( (*vector)->data ); /* free allocated data */
  LALFree( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN( status );
}


















void U2CreateVector ( Status *status, UINT2Vector **vector, UINT4 length ) 
{
  /* 
   * Initialize status structure
   */

  INITSTATUS( status, "U2CreateVector", VECTORFACTORIESC );	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT( length > 0, status, CREATEVECTOR_ELENGTH, CREATEVECTOR_MSGELENGTH );

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT( vector != NULL, status, CREATEVECTOR_EVPTR, CREATEVECTOR_MSGEVPTR );

  ASSERT( *vector == NULL, status, CREATEVECTOR_EUPTR, CREATEVECTOR_MSGEUPTR );

  /*
   * Allocate pointer
   */

  *vector = ( UINT2Vector * ) LALMalloc( sizeof( UINT2Vector ) );
  ASSERT( *vector != NULL, status,
          CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC );

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = ( UINT2 * ) LALMalloc( length*sizeof( UINT2 ) );

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree( *vector );
    ABORT( status, CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC );
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN( status );
}
















void U2DestroyVector ( Status *status, UINT2Vector **vector )
{
  /* 
   * Initialize status
   */

  INITSTATUS( status, "U2DestroyVector", VECTORFACTORIESC );	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT( vector != NULL, status, DESTROYVECTOR_EVPTR, DESTROYVECTOR_MSGEVPTR );

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT( *vector != NULL, status, DESTROYVECTOR_EUPTR, DESTROYVECTOR_MSGEUPTR );

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT( (*vector)->data != NULL, status,
          DESTROYVECTOR_EDPTR, DESTROYVECTOR_MSGEDPTR );

  /* Ok, now let's free allocated storage */

  LALFree( (*vector)->data ); /* free allocated data */
  LALFree( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN( status );
}


















void U4CreateVector ( Status *status, UINT4Vector **vector, UINT4 length ) 
{
  /* 
   * Initialize status structure
   */

  INITSTATUS( status, "U4CreateVector", VECTORFACTORIESC );	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT( length > 0, status, CREATEVECTOR_ELENGTH, CREATEVECTOR_MSGELENGTH );

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT( vector != NULL, status, CREATEVECTOR_EVPTR, CREATEVECTOR_MSGEVPTR );

  ASSERT( *vector == NULL, status, CREATEVECTOR_EUPTR, CREATEVECTOR_MSGEUPTR );

  /*
   * Allocate pointer
   */

  *vector = ( UINT4Vector * ) LALMalloc( sizeof( UINT4Vector ) );
  ASSERT( *vector != NULL, status,
          CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC );

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = ( UINT4 * ) LALMalloc( length*sizeof( UINT4 ) );

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree( *vector );
    ABORT( status, CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC );
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN( status );
}
















void U4DestroyVector ( Status *status, UINT4Vector **vector )
{
  /* 
   * Initialize status
   */

  INITSTATUS( status, "U4DestroyVector", VECTORFACTORIESC );	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT( vector != NULL, status, DESTROYVECTOR_EVPTR, DESTROYVECTOR_MSGEVPTR );

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT( *vector != NULL, status, DESTROYVECTOR_EUPTR, DESTROYVECTOR_MSGEUPTR );

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT( (*vector)->data != NULL, status,
          DESTROYVECTOR_EDPTR, DESTROYVECTOR_MSGEDPTR );

  /* Ok, now let's free allocated storage */

  LALFree( (*vector)->data ); /* free allocated data */
  LALFree( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN( status );
}


















void U8CreateVector ( Status *status, UINT8Vector **vector, UINT4 length ) 
{
  /* 
   * Initialize status structure
   */

  INITSTATUS( status, "U8CreateVector", VECTORFACTORIESC );	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT( length > 0, status, CREATEVECTOR_ELENGTH, CREATEVECTOR_MSGELENGTH );

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT( vector != NULL, status, CREATEVECTOR_EVPTR, CREATEVECTOR_MSGEVPTR );

  ASSERT( *vector == NULL, status, CREATEVECTOR_EUPTR, CREATEVECTOR_MSGEUPTR );

  /*
   * Allocate pointer
   */

  *vector = ( UINT8Vector * ) LALMalloc( sizeof( UINT8Vector ) );
  ASSERT( *vector != NULL, status,
          CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC );

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = ( UINT8 * ) LALMalloc( length*sizeof( UINT8 ) );

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree( *vector );
    ABORT( status, CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC );
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN( status );
}
















void U8DestroyVector ( Status *status, UINT8Vector **vector )
{
  /* 
   * Initialize status
   */

  INITSTATUS( status, "U8DestroyVector", VECTORFACTORIESC );	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT( vector != NULL, status, DESTROYVECTOR_EVPTR, DESTROYVECTOR_MSGEVPTR );

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT( *vector != NULL, status, DESTROYVECTOR_EUPTR, DESTROYVECTOR_MSGEUPTR );

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT( (*vector)->data != NULL, status,
          DESTROYVECTOR_EDPTR, DESTROYVECTOR_MSGEDPTR );

  /* Ok, now let's free allocated storage */

  LALFree( (*vector)->data ); /* free allocated data */
  LALFree( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN( status );
}


















void CHARCreateVector ( Status *status, CHARVector **vector, UINT4 length ) 
{
  /* 
   * Initialize status structure
   */

  INITSTATUS( status, "CHARCreateVector", VECTORFACTORIESC );	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT( length > 0, status, CREATEVECTOR_ELENGTH, CREATEVECTOR_MSGELENGTH );

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT( vector != NULL, status, CREATEVECTOR_EVPTR, CREATEVECTOR_MSGEVPTR );

  ASSERT( *vector == NULL, status, CREATEVECTOR_EUPTR, CREATEVECTOR_MSGEUPTR );

  /*
   * Allocate pointer
   */

  *vector = ( CHARVector * ) LALMalloc( sizeof( CHARVector ) );
  ASSERT( *vector != NULL, status,
          CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC );

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = ( CHAR * ) LALMalloc( length*sizeof( CHAR ) );

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree( *vector );
    ABORT( status, CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC );
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN( status );
}
















void CHARDestroyVector ( Status *status, CHARVector **vector )
{
  /* 
   * Initialize status
   */

  INITSTATUS( status, "CHARDestroyVector", VECTORFACTORIESC );	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT( vector != NULL, status, DESTROYVECTOR_EVPTR, DESTROYVECTOR_MSGEVPTR );

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT( *vector != NULL, status, DESTROYVECTOR_EUPTR, DESTROYVECTOR_MSGEUPTR );

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT( (*vector)->data != NULL, status,
          DESTROYVECTOR_EDPTR, DESTROYVECTOR_MSGEDPTR );

  /* Ok, now let's free allocated storage */

  LALFree( (*vector)->data ); /* free allocated data */
  LALFree( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN( status );
}


















void CreateVector ( Status *status, REAL4Vector **vector, UINT4 length ) 
{
  /* 
   * Initialize status structure
   */

  INITSTATUS( status, "CreateVector", VECTORFACTORIESC );	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT( length > 0, status, CREATEVECTOR_ELENGTH, CREATEVECTOR_MSGELENGTH );

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT( vector != NULL, status, CREATEVECTOR_EVPTR, CREATEVECTOR_MSGEVPTR );

  ASSERT( *vector == NULL, status, CREATEVECTOR_EUPTR, CREATEVECTOR_MSGEUPTR );

  /*
   * Allocate pointer
   */

  *vector = ( REAL4Vector * ) LALMalloc( sizeof( REAL4Vector ) );
  ASSERT( *vector != NULL, status,
          CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC );

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = ( REAL4 * ) LALMalloc( length*sizeof( REAL4 ) );

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree( *vector );
    ABORT( status, CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC );
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN( status );
}
















void DestroyVector ( Status *status, REAL4Vector **vector )
{
  /* 
   * Initialize status
   */

  INITSTATUS( status, "DestroyVector", VECTORFACTORIESC );	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT( vector != NULL, status, DESTROYVECTOR_EVPTR, DESTROYVECTOR_MSGEVPTR );

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT( *vector != NULL, status, DESTROYVECTOR_EUPTR, DESTROYVECTOR_MSGEUPTR );

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT( (*vector)->data != NULL, status,
          DESTROYVECTOR_EDPTR, DESTROYVECTOR_MSGEDPTR );

  /* Ok, now let's free allocated storage */

  LALFree( (*vector)->data ); /* free allocated data */
  LALFree( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN( status );
}

