/*----------------------------------------------------------------------- 
 * 
 * File Name: IntegrateTest.c
 * 
 * Revision: $Id: InterpolateTest.c,v 1.4 2000/04/21 16:25:21 jolien Exp $
 * 
 *-----------------------------------------------------------------------
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "LALConfig.h"

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#include "LALStdlib.h"
#include "AVFactories.h"
#include "Interpolate.h"

#define CODES_(x) #x
#define CODES(x) CODES_(x)

NRCSID (MAIN, "$Id: InterpolateTest.c,v 1.4 2000/04/21 16:25:21 jolien Exp $");

extern char *optarg;
extern int   optind;

int   debuglevel = 0;
int   verbose    = 0;

static void Usage (const char *program, int exitflag);

static void ParseOptions (int argc, char *argv[]);

static void TestStatus (Status *status, const char *expectCodes, int exitCode);

static void ClearStatus (Status *status);

int main (int argc, char *argv[])
{
  enum {ArraySize = 10};

  static Status       status;
  static REAL4Vector *x;
  static REAL4Vector *y;
  static REAL8Vector *xx;
  static REAL8Vector *yy;

  SInterpolateOut sintout;
  SInterpolatePar sintpar;
  DInterpolateOut dintout;
  DInterpolatePar dintpar;

  int i;

  ParseOptions (argc, argv);

  SCreateVector (&status, &x, ArraySize);
  TestStatus (&status, CODES(0), 1);
  SCreateVector (&status, &y, ArraySize);
  TestStatus (&status, CODES(0), 1);
  DCreateVector (&status, &xx, ArraySize);
  TestStatus (&status, CODES(0), 1);
  DCreateVector (&status, &yy, ArraySize);
  TestStatus (&status, CODES(0), 1);

  printf ("Initial data:\n");
  printf ("y = P(x) = 7 - 8x + 2x^2 + 2x^3 - x^4\n");
  printf ("-----------------\n");
  printf ("x\ty\n");
  printf ("=================\n");
  for (i = 0; i < ArraySize; ++i)
  {
    REAL4 xi = x->data[i] = xx->data[i] = i*i;
    y->data[i] = yy->data[i] = 7 + xi*(-8 + xi*(2 + xi*(2 - xi)));
    printf ("%.0f\t%.0f\n", x->data[i], y->data[i]);
  }
  printf ("-----------------\n");

  printf ("\nInterpolate to x = 0.3:\n");
  printf ("---------------------------------\n");
  printf ("order\ty\t\tdy\n");
  printf ("=================================\n");
  sintpar.x = x->data;
  sintpar.y = y->data;
  dintpar.x = xx->data;
  dintpar.y = yy->data;
  for (i = 2; i < ArraySize; ++i)
  {
    sintpar.n = i;
    dintpar.n = i;
    SPolynomialInterpolation (&status, &sintout, 0.3, &sintpar);
    TestStatus (&status, CODES(0), 1);
    DPolynomialInterpolation (&status, &dintout, 0.3, &dintpar);
    TestStatus (&status, CODES(0), 1);
    printf ("%d\t%f\t%f\t%f\t%f\n", i - 1,
            sintout.y, sintout.dy, dintout.y, dintout.dy);
  }
  printf ("---------------------------------\n");

  printf ("\nExtrapolate to x = -0.3:\n");
  printf ("---------------------------------\n");
  printf ("order\ty\t\tdy\n");
  printf ("=================================\n");
  sintpar.x = x->data;
  sintpar.y = y->data;
  dintpar.x = xx->data;
  dintpar.y = yy->data;
  for (i = 2; i < ArraySize; ++i)
  {
    sintpar.n = i;
    dintpar.n = i;
    SPolynomialInterpolation (&status, &sintout, -0.3, &sintpar);
    TestStatus (&status, CODES(0), 1);
    DPolynomialInterpolation (&status, &dintout, -0.3, &dintpar);
    TestStatus (&status, CODES(0), 1);
    printf ("%d\t%f\t%f\t%f\t%f\n", i - 1,
            sintout.y, sintout.dy, dintout.y, dintout.dy);
  }
  printf ("---------------------------------\n");


  SDestroyVector (&status, &x);
  TestStatus (&status, CODES(0), 1);
  SDestroyVector (&status, &y);
  TestStatus (&status, CODES(0), 1);
  DDestroyVector (&status, &xx);
  TestStatus (&status, CODES(0), 1);
  DDestroyVector (&status, &yy);
  TestStatus (&status, CODES(0), 1);

  LALCheckMemoryLeaks ();

  SCreateVector (&status, &x, ArraySize);
  TestStatus (&status, CODES(0), 1);
  SCreateVector (&status, &y, ArraySize);
  TestStatus (&status, CODES(0), 1);
  DCreateVector (&status, &xx, ArraySize);
  TestStatus (&status, CODES(0), 1);
  DCreateVector (&status, &yy, ArraySize);
  TestStatus (&status, CODES(0), 1);


  printf ("\nCheck error conditions:\n");

  printf ("\nNull pointer:\r");
  SPolynomialInterpolation (&status, NULL, -0.3, &sintpar);
  TestStatus (&status, CODES(INTERPOLATE_ENULL), 1);
  DPolynomialInterpolation (&status, NULL, -0.3, &dintpar);
  TestStatus (&status, CODES(INTERPOLATE_ENULL), 1);
  printf ("Null pointer check passed.\n");

  printf ("\nNull pointer:\r");
  SPolynomialInterpolation (&status, &sintout, -0.3, NULL);
  TestStatus (&status, CODES(INTERPOLATE_ENULL), 1);
  DPolynomialInterpolation (&status, &dintout, -0.3, NULL);
  TestStatus (&status, CODES(INTERPOLATE_ENULL), 1);
  printf ("Null pointer check passed.\n");

  sintpar.x = NULL;
  dintpar.x = NULL;
  printf ("\nNull pointer:\r");
  SPolynomialInterpolation (&status, &sintout, -0.3, &sintpar);
  TestStatus (&status, CODES(INTERPOLATE_ENULL), 1);
  DPolynomialInterpolation (&status, &dintout, -0.3, &dintpar);
  TestStatus (&status, CODES(INTERPOLATE_ENULL), 1);
  printf ("Null pointer check passed.\n");

  sintpar.x = x->data;
  sintpar.y = NULL;
  dintpar.x = xx->data;
  dintpar.y = NULL;
  printf ("\nNull pointer:\r");
  SPolynomialInterpolation (&status, &sintout, -0.3, &sintpar);
  TestStatus (&status, CODES(INTERPOLATE_ENULL), 1);
  DPolynomialInterpolation (&status, &dintout, -0.3, &dintpar);
  TestStatus (&status, CODES(INTERPOLATE_ENULL), 1);
  printf ("Null pointer check passed.\n");

  sintpar.y = y->data;
  sintpar.n = 1;
  dintpar.y = yy->data;
  dintpar.n = 1;
  printf ("\nInvalid size:\r");
  SPolynomialInterpolation (&status, &sintout, -0.3, &sintpar);
  TestStatus (&status, CODES(INTERPOLATE_ESIZE), 1);
  DPolynomialInterpolation (&status, &dintout, -0.3, &dintpar);
  TestStatus (&status, CODES(INTERPOLATE_ESIZE), 1);
  printf ("Invalid size check passed.\n");

  x->data[1]  = x->data[0]  = 2;
  xx->data[1] = xx->data[0] = 2;
  sintpar.n = 3;
  dintpar.n = 3;
  printf ("\nZero divide:\r");
  SPolynomialInterpolation (&status, &sintout, -0.3, &sintpar);
  TestStatus (&status, CODES(INTERPOLATE_EZERO), 1);
  DPolynomialInterpolation (&status, &dintout, -0.3, &dintpar);
  TestStatus (&status, CODES(INTERPOLATE_EZERO), 1);
  printf ("Zero divide check passed.\n");


  SDestroyVector (&status, &x);
  TestStatus (&status, CODES(0), 1);
  SDestroyVector (&status, &y);
  TestStatus (&status, CODES(0), 1);
  DDestroyVector (&status, &xx);
  TestStatus (&status, CODES(0), 1);
  DDestroyVector (&status, &yy);
  TestStatus (&status, CODES(0), 1);

  return 0;
}


/*
 * TestStatus ()
 *
 * Routine to check that the status code status->statusCode agrees with one of
 * the codes specified in the space-delimited string ignored; if not,
 * exit to the system with code exitcode.
 *
 */
static void
TestStatus (Status *status, const char *ignored, int exitcode)
{
  char  str[64];
  char *tok;

  if (verbose)
  {
    REPORTSTATUS (status);
  }

  if (strncpy (str, ignored, sizeof (str)))
  {
    if ((tok = strtok (str, " ")))
    {
      do
      {
        if (status->statusCode == atoi (tok))
        {
          return;
        }
      }
      while ((tok = strtok (NULL, " ")));
    }
    else
    {
      if (status->statusCode == atoi (tok))
      {
        return;
      }
    }
  }

  fprintf (stderr, "\nExiting to system with code %d\n", exitcode);
  exit (exitcode);
}


/*
 *
 * ClearStatus ()
 *
 * Recursively applies DETATCHSTATUSPTR() to status structure to destroy
 * linked list of statuses.
 *
 */
void
ClearStatus (Status *status)
{
  if (status->statusPtr)
  {
    ClearStatus      (status->statusPtr);
    DETATCHSTATUSPTR (status);
  }
}


/*
 * Usage ()
 *
 * Prints a usage message for program program and exits with code exitcode.
 *
 */
static void
Usage (const char *program, int exitcode)
{
  fprintf (stderr, "Usage: %s [options]\n", program);
  fprintf (stderr, "Options:\n");
  fprintf (stderr, "  -h         print this message\n");
  fprintf (stderr, "  -q         quiet: run silently\n");
  fprintf (stderr, "  -v         verbose: print extra information\n");
  fprintf (stderr, "  -d level   set debuglevel to level\n");
  exit (exitcode);
}


/*
 * ParseOptions ()
 *
 * Parses the argc - 1 option strings in argv[].
 *
 */
static void
ParseOptions (int argc, char *argv[])
{
  while (1)
  {
    int c = -1;

    c = getopt (argc, argv, "hqvd:");
    if (c == -1)
    {
      break;
    }

    switch (c)
    {
      case 'd': /* set debug level */
        debuglevel = atoi (optarg);
        break;

      case 'v': /* verbose */
        ++verbose;
        break;

      case 'q': /* quiet: run silently */
        freopen ("/dev/null", "w", stderr);
        freopen ("/dev/null", "w", stdout);
        break;

      case 'h':
        Usage (argv[0], 0);
        break;

      default:
        Usage (argv[0], 1);
    }

  }

  if (optind < argc)
  {
    Usage (argv[0], 1);
  }

  return;
}


